// $Id: ZIP.cc,v 1.1.4.2 2006/05/31 21:49:29 sommer Exp $
//
// Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#include "ZIP.h"

#ifdef HAVE_LIBZ

ZIP_Analyzer::ZIP_Analyzer(Connection* conn, bool orig)
: TCP_SupportAnalyzer(AnalyzerTag::Zip, conn, orig)
	{
	zip = 0;
	zip_status = Z_OK;
	}

ZIP_Analyzer::~ZIP_Analyzer()
	{
	delete zip;
	}

void ZIP_Analyzer::Done()
	{
	Analyzer::Done();

	if ( zip )
		inflateEnd(zip);
	}

void ZIP_Analyzer::DeliverStream(int len, const u_char* data, bool orig)
	{
	TCP_SupportAnalyzer::DeliverStream(len, data, orig);

	if ( ! len || zip_status != Z_OK )
		return;

	static unsigned int unzip_size = 4096;
	Bytef unzipbuf[unzip_size];

	if ( ! zip )
		{
		zip = new z_stream;

		zip->zalloc = 0;
		zip->zfree = 0;
		zip->opaque = 0;

		zip->next_out = unzipbuf;
		zip->avail_out = unzip_size;

		zip->next_in = (Bytef*) data;
		zip->avail_in = len;

		// "15" here means maximum compression.  "32" is
		// a gross overload hack that means "check it for
		// whether it's a gzip file".  Sheesh.
		zip_status = inflateInit2(zip, 15 + 32);
		if ( zip_status != Z_OK )
			{
			Weird("inflate_data_failed");
			return;
			}
		}
	else
		{
		zip->next_in = (Bytef*) data;
		zip->avail_in = len;
		}

	do
		{
		zip->next_out = unzipbuf;
		zip->avail_out = unzip_size;
		zip_status = inflate(zip, Z_SYNC_FLUSH);

		if ( zip_status != Z_STREAM_END &&
		     zip_status != Z_OK &&
		     zip_status != Z_BUF_ERROR )
			{
			Weird("inflate_failed");
			inflateEnd(zip);
			break;
			}

		int have = unzip_size - zip->avail_out;
		if ( have )
			ForwardStream(have, unzipbuf, IsOrig());

		if ( zip_status == Z_STREAM_END )
			{
			inflateEnd(zip);
			delete zip;
			zip = 0;
			break;
			}

		zip_status = Z_OK;
		}
	while ( zip->avail_out == 0 );
	}
#endif
